@@ -812,6 +812,8 @@ static void cellarray(double xmin, double xmax, double ymin, double ymax, int dx
 
 static void set_clip_rect(int tnr)
 {
+  double x, y, w, h;
+
   cairo_reset_clip(p->cr);
 
   if (gkss->clip_tnr != 0 || gkss->clip == GKS_K_CLIP)
@@ -820,8 +822,22 @@ static void set_clip_rect(int tnr)
         {
           tnr = gkss->clip_tnr;
         }
-      cairo_rectangle(p->cr, p->rect[tnr][0][0], p->rect[tnr][0][1], p->rect[tnr][1][0] - p->rect[tnr][0][0],
-                      p->rect[tnr][1][1] - p->rect[tnr][0][1]);
+      x = p->rect[tnr][0][0];
+      y = p->rect[tnr][0][1];
+      w = p->rect[tnr][1][0] - p->rect[tnr][0][0];
+      h = p->rect[tnr][1][1] - p->rect[tnr][0][1];
+      if (gkss->clip_region == GKS_K_REGION_ELLIPSE)
+        {
+          cairo_save(p->cr);
+          cairo_translate(p->cr, x + 0.5 * w, y + 0.5 * h);
+          cairo_scale(p->cr, 1., h / w);
+          cairo_arc(p->cr, 0., 0., w * 0.5, 0, 2 * M_PI);
+          cairo_restore(p->cr);
+        }
+      else
+        {
+          cairo_rectangle(p->cr, x, y, w, h);
+        }
       cairo_clip(p->cr);
     }
 }
