@@ -7,7 +7,7 @@
 #include "args_int.h"
 #include "interaction_int.h"
 #include "plot_int.h"
-#include "util_int.h"
+#include "utilcpp_int.hxx"
 #include "gr.h"
 #include "grm/dom_render/render.hxx"
 #include <grm/dom_render/graphics_tree/util.hxx>
@@ -357,6 +357,89 @@ err_t tooltip_list_entry_delete(tooltip_list_entry_t entry)
 
 /* ------------------------- user interaction ----------------------------------------------------------------------- */
 
+static void moveTransformationHelper(const std::shared_ptr<GRM::Element> &element, double ndc_x, double ndc_y,
+                                     double xshift, double yshift, bool is_movable)
+{
+  double tmp = 0, x_with_shift, y_with_shift, x_ndc, y_ndc;
+  double old_x_shift = 0, old_y_shift = 0, wc_x_shift, wc_y_shift;
+  int width, height, max_width_height;
+  std::string post_fix = "_wc";
+  std::vector<std::string> ndc_transformation_elems = {"figure",
+                                                       "plot",
+                                                       "colorbar",
+                                                       "labels_group",
+                                                       "titles_3d",
+                                                       "text",
+                                                       "y_tick_label_group",
+                                                       "x_tick_label_group",
+                                                       "layout_grid_element",
+                                                       "layout_grid",
+                                                       "central_region",
+                                                       "side_region",
+                                                       "marginal_heatmap_plot"};
+
+  GRM::Render::getFigureSize(&width, &height, nullptr, nullptr);
+  max_width_height = grm_max(width, height);
+
+  if (std::find(ndc_transformation_elems.begin(), ndc_transformation_elems.end(), element->localName()) !=
+      ndc_transformation_elems.end())
+    post_fix = "_ndc";
+
+  x_with_shift = ndc_x + (double)xshift / max_width_height;
+  x_ndc = ndc_x;
+  y_with_shift = ndc_y + (double)yshift / max_width_height;
+  y_ndc = ndc_y;
+
+  gr_ndctowc(&x_with_shift, &y_with_shift);
+  gr_ndctowc(&x_ndc, &y_ndc);
+
+  wc_x_shift = x_with_shift - x_ndc;
+  wc_y_shift = y_ndc - y_with_shift;
+
+  if (element->hasAttribute("x_shift" + post_fix))
+    old_x_shift = static_cast<double>(element->getAttribute("x_shift" + post_fix));
+  if (element->hasAttribute("y_shift" + post_fix))
+    old_y_shift = static_cast<double>(element->getAttribute("y_shift" + post_fix));
+
+  if (xshift != 0)
+    {
+      if (post_fix == "_wc")
+        {
+          if (is_movable && element->localName() == "polyline" &&
+              static_cast<std::string>(element->getAttribute("name")) == "integral_left")
+            {
+              auto old_int_lim_low = static_cast<double>(element->parentElement()->getAttribute("int_lim_low"));
+              element->parentElement()->setAttribute("int_lim_low", old_int_lim_low + wc_x_shift);
+            }
+          else if (is_movable && element->localName() == "polyline" &&
+                   static_cast<std::string>(element->getAttribute("name")) == "integral_right")
+            {
+              auto old_int_lim_high = static_cast<double>(element->parentElement()->getAttribute("int_lim_high"));
+              element->parentElement()->setAttribute("int_lim_high", old_int_lim_high + wc_x_shift);
+            }
+          else
+            {
+              element->setAttribute("x_shift" + post_fix, old_x_shift + wc_x_shift);
+            }
+        }
+      else
+        {
+          element->setAttribute("x_shift" + post_fix, old_x_shift + (double)xshift / max_width_height);
+        }
+    }
+  if (yshift != 0)
+    {
+      if (post_fix == "_wc")
+        {
+          element->setAttribute("y_shift" + post_fix, old_y_shift + wc_y_shift);
+        }
+      else
+        {
+          element->setAttribute("y_shift" + post_fix, old_y_shift + (double)yshift / max_width_height);
+        }
+    }
+}
+
 int grm_input(const grm_args_t *input_args)
 {
   /*
@@ -385,14 +468,14 @@ int grm_input(const grm_args_t *input_args)
   int x, y, x1, y1, x2, y2;
   double viewport_mid_x, viewport_mid_y;
   bool clear_locked_state = false;
-
+  int selection_status;
   logger((stderr, "Processing input\n"));
 
-  GRM::Render::get_figure_size(&width, &height, nullptr, nullptr);
+  GRM::Render::getFigureSize(&width, &height, nullptr, nullptr);
   max_width_height = grm_max(width, height);
   logger((stderr, "Using size (%d, %d)\n", width, height));
 
-  auto marginal_heatmap = grm_get_document_root()->querySelectorsAll("series_marginal_heatmap");
+  auto marginal_heatmap = grm_get_document_root()->querySelectorsAll("marginal_heatmap_plot");
   if (!marginal_heatmap.empty())
     {
       if (marginal_heatmap[0]->hasAttribute("x_ind")) marginal_heatmap[0]->setAttribute("x_ind", -1);
@@ -414,6 +497,11 @@ int grm_input(const grm_args_t *input_args)
       logger((stderr, "x: %d, y: %d, ndc_x: %lf, ndc_y: %lf\n", x, y, ndc_x, ndc_y));
 
       auto subplot_element = get_subplot_from_ndc_point_using_dom(ndc_x, ndc_y);
+      if (subplot_element == nullptr && grm_args_values(input_args, "move_selection", "i", &selection_status))
+        {
+          // needed so the cursor doesn't need to be inside the window for gredit element moving
+          subplot_element = grm_get_document_root()->querySelectors("plot[plot_group]");
+        }
 
       if (grm_args_values(input_args, "key", "s", &key))
         {
@@ -423,8 +511,20 @@ int grm_input(const grm_args_t *input_args)
             {
               if (subplot_element != nullptr)
                 {
-                  logger((stderr, "Reset single subplot coordinate ranges\n"));
-                  subplot_element->setAttribute("reset_ranges", 1);
+                  auto coordinate_system = subplot_element->querySelectors("coordinate_system");
+                  if (coordinate_system->hasAttribute("plot_type") &&
+                      static_cast<std::string>(coordinate_system->getAttribute("plot_type")) == "2d")
+                    {
+                      logger((stderr, "Reset single subplot coordinate ranges\n"));
+                      subplot_element->setAttribute("reset_ranges", 1);
+                    }
+                  if (coordinate_system->hasAttribute("plot_type") &&
+                      static_cast<std::string>(coordinate_system->getAttribute("plot_type")) == "3d")
+                    {
+                      logger((stderr, "Reset single subplot coordinate rotation\n"));
+                      auto central_region = subplot_element->querySelectors("central_region");
+                      central_region->setAttribute("reset_rotation", 1);
+                    }
                 }
               else
                 {
@@ -443,10 +543,11 @@ int grm_input(const grm_args_t *input_args)
           int movable_status;
           std::string kind;
           double viewport[4];
-          viewport[0] = static_cast<double>(subplot_element->getAttribute("viewport_x_min"));
-          viewport[1] = static_cast<double>(subplot_element->getAttribute("viewport_x_max"));
-          viewport[2] = static_cast<double>(subplot_element->getAttribute("viewport_y_min"));
-          viewport[3] = static_cast<double>(subplot_element->getAttribute("viewport_y_max"));
+          auto central_region = subplot_element->querySelectors("central_region");
+          viewport[0] = static_cast<double>(central_region->getAttribute("viewport_x_min"));
+          viewport[1] = static_cast<double>(central_region->getAttribute("viewport_x_max"));
+          viewport[2] = static_cast<double>(central_region->getAttribute("viewport_y_min"));
+          viewport[3] = static_cast<double>(central_region->getAttribute("viewport_y_max"));
 
           kind = static_cast<std::string>(subplot_element->getAttribute("kind"));
           if (kind == "marginal_heatmap")
@@ -501,25 +602,27 @@ int grm_input(const grm_args_t *input_args)
                   y_0 = y_min;
                 }
 
-              GRM::Render::processViewport(subplot_element);
               GRM::Render::processLimits(subplot_element);
-              GRM::Render::processWindow(subplot_element);
+              GRM::Render::processWindow(central_region);
 
               gr_savestate();
               for (const auto &elem : subplot_element->parentElement()->children())
                 {
-                  if (elem->hasAttribute("viewport_x_min"))
+                  if (elem->hasAttribute("scale"))
                     {
-                      gr_setviewport(
-                          (double)elem->getAttribute("viewport_x_min"), (double)elem->getAttribute("viewport_x_max"),
-                          (double)elem->getAttribute("viewport_y_min"), (double)elem->getAttribute("viewport_y_max"));
-                      gr_setwindow(
-                          (double)elem->getAttribute("window_x_min"), (double)elem->getAttribute("window_x_max"),
-                          (double)elem->getAttribute("window_y_min"), (double)elem->getAttribute("window_y_max"));
+                      gr_setviewport(static_cast<double>(central_region->getAttribute("viewport_x_min")),
+                                     static_cast<double>(central_region->getAttribute("viewport_x_max")),
+                                     static_cast<double>(central_region->getAttribute("viewport_y_min")),
+                                     static_cast<double>(central_region->getAttribute("viewport_y_max")));
+                      gr_setwindow(static_cast<double>(central_region->getAttribute("window_x_min")),
+                                   static_cast<double>(central_region->getAttribute("window_x_max")),
+                                   static_cast<double>(central_region->getAttribute("window_y_min")),
+                                   static_cast<double>(central_region->getAttribute("window_y_max")));
                       gr_setscale(static_cast<int>(elem->getAttribute("scale")));
                       break;
                     }
                 }
+              GRM::Render::calculateCharHeight(central_region);
 
               gr_wctondc(&x_0, &y_0);
               gr_wctondc(&x_end, &y_end);
@@ -564,7 +667,7 @@ int grm_input(const grm_args_t *input_args)
               if (static_cast<int>(subplot_element->getAttribute("x_log")))
                 {
                   double tmp = 0;
-                  xind_d = x_offset;
+                  xind_d = 0;
                   for (int j = 0; j < x_length - x_offset; j++)
                     {
                       if (tmp + x_steps[j] > (x - x_0)) break;
@@ -577,7 +680,7 @@ int grm_input(const grm_args_t *input_args)
               if (static_cast<int>(subplot_element->getAttribute("y_log")))
                 {
                   double tmp = 0;
-                  yind_d = y_offset;
+                  yind_d = 0;
                   for (int j = 0; j < y_length - y_offset; j++)
                     {
                       if (tmp + y_steps[j] < (y - y_0)) break;
@@ -597,49 +700,56 @@ int grm_input(const grm_args_t *input_args)
                   yind = (int)yind_d;
                 }
 
-              auto old_xind = static_cast<int>(current_series->parentElement()->getAttribute("x_ind"));
-              auto old_yind = static_cast<int>(current_series->parentElement()->getAttribute("y_ind"));
-              current_series->parentElement()->setAttribute("x_ind", xind);
-              current_series->parentElement()->setAttribute("y_ind", yind);
-              if (static_cast<std::string>(current_series->parentElement()->getAttribute("marginal_heatmap_kind")) ==
-                      "line" &&
+              auto old_xind = static_cast<int>(marginal_heatmap[0]->getAttribute("x_ind"));
+              auto old_yind = static_cast<int>(marginal_heatmap[0]->getAttribute("y_ind"));
+              marginal_heatmap[0]->setAttribute("x_ind", xind);
+              marginal_heatmap[0]->setAttribute("y_ind", yind);
+              if (static_cast<std::string>(marginal_heatmap[0]->getAttribute("marginal_heatmap_kind")) == "line" &&
                   ((old_xind == -1 || old_yind == -1) && xind != -1 && yind != -1))
-                current_series->parentElement()->setAttribute("_update_required", true);
+                marginal_heatmap[0]->setAttribute("_update_required", true);
 
-              for (auto &child : current_series->parentElement()->children())
+              for (auto &side_region : marginal_heatmap[0]->children())
                 {
-                  std::string childKind = static_cast<std::string>(child->getAttribute("kind"));
-                  if (childKind == "hist")
+                  if (side_region->localName() == "side_region")
                     {
-                      // reset bar colors
-                      bool is_horizontal = static_cast<std::string>(child->getAttribute("orientation")) == "horizontal";
-
-                      // bar level
-                      for (auto &bars : child->children())
+                      for (auto &child : side_region->children())
                         {
-                          // fill- and draw_rect level
-                          for (auto &childSeries : bars->children())
+                          std::string childKind = static_cast<std::string>(child->getAttribute("kind"));
+                          if (childKind == "hist")
                             {
-                              auto groups = childSeries->children(); // innerFillGroup and outerFillGroup
-                              std::shared_ptr<GRM::Element> innerFillGroup;
-                              if (groups.size() == 2)
-                                {
-                                  innerFillGroup = groups[0];
-                                }
-                              else
-                                {
-                                  // no fillGroups?
-                                  break;
-                                }
+                              // reset bar colors
+                              bool is_horizontal =
+                                  static_cast<std::string>(child->getAttribute("orientation")) == "horizontal";
 
-                              int fillColorInd = static_cast<int>(innerFillGroup->getAttribute("fill_color_ind"));
-                              if (xind != -1)
+                              // bar level
+                              for (auto &bars : child->children())
                                 {
-                                  innerFillGroup->children()[xind]->removeAttribute("fill_color_ind");
-                                }
-                              if (yind != -1)
-                                {
-                                  innerFillGroup->children()[yind]->removeAttribute("fill_color_ind");
+                                  // fill- and draw_rect level
+                                  for (auto &childSeries : bars->children())
+                                    {
+                                      auto groups = childSeries->children(); // innerFillGroup and outerFillGroup
+                                      std::shared_ptr<GRM::Element> innerFillGroup;
+                                      if (groups.size() == 2)
+                                        {
+                                          innerFillGroup = groups[0];
+                                        }
+                                      else
+                                        {
+                                          // no fillGroups?
+                                          break;
+                                        }
+
+                                      int fillColorInd =
+                                          static_cast<int>(innerFillGroup->getAttribute("fill_color_ind"));
+                                      if (xind != -1)
+                                        {
+                                          innerFillGroup->children()[xind]->removeAttribute("fill_color_ind");
+                                        }
+                                      if (yind != -1)
+                                        {
+                                          innerFillGroup->children()[yind]->removeAttribute("fill_color_ind");
+                                        }
+                                    }
                                 }
                             }
                         }
@@ -652,7 +762,7 @@ int grm_input(const grm_args_t *input_args)
             {
               double focus_x, focus_y;
 
-              if (str_equals_any(kind.c_str(), 7, "wireframe", "surface", "plot3", "scatter3", "trisurface", "volume",
+              if (str_equals_any(kind, "wireframe", "surface", "plot3", "scatter3", "trisurface", "volume",
                                  "isosurface"))
                 {
                   /*
@@ -679,7 +789,7 @@ int grm_input(const grm_args_t *input_args)
             {
               double focus_x, focus_y;
 
-              if (str_equals_any(kind.c_str(), 7, "wireframe", "surface", "plot3", "scatter3", "trisurface", "volume",
+              if (str_equals_any(kind, "wireframe", "surface", "plot3", "scatter3", "trisurface", "volume",
                                  "isosurface"))
                 {
                   /*
@@ -697,18 +807,49 @@ int grm_input(const grm_args_t *input_args)
                   subplot_element->append(panzoom_element);
                   subplot_element->setAttribute("panzoom", true);
                 }
-
               return 1;
             }
 
           if (grm_args_values(input_args, "x_shift", "i", &xshift) &&
               grm_args_values(input_args, "y_shift", "i", &yshift) &&
-              !grm_args_values(input_args, "movable_state", "i", &movable_status))
+              grm_args_values(input_args, "move_selection", "i", &selection_status))
+            {
+              GRM::Render::processLimits(subplot_element);
+              GRM::Render::processWindow(central_region);
+
+              gr_savestate();
+              for (const auto &elem : subplot_element->parentElement()->children())
+                {
+                  if (elem->hasAttribute("scale"))
+                    {
+                      gr_setviewport(static_cast<double>(central_region->getAttribute("viewport_x_min")),
+                                     static_cast<double>(central_region->getAttribute("viewport_x_max")),
+                                     static_cast<double>(central_region->getAttribute("viewport_y_min")),
+                                     static_cast<double>(central_region->getAttribute("viewport_y_max")));
+                      gr_setwindow(static_cast<double>(central_region->getAttribute("window_x_min")),
+                                   static_cast<double>(central_region->getAttribute("window_x_max")),
+                                   static_cast<double>(central_region->getAttribute("window_y_min")),
+                                   static_cast<double>(central_region->getAttribute("window_y_max")));
+                      gr_setscale(static_cast<int>(elem->getAttribute("scale")));
+                      break;
+                    }
+                }
+              GRM::Render::calculateCharHeight(central_region);
+
+              for (const auto &selection : grm_get_document_root()->querySelectorsAll("[_selected=1]"))
+                {
+                  moveTransformationHelper(selection, ndc_x, ndc_y, xshift, yshift, false);
+                }
+              gr_restorestate();
+            }
+          else if (grm_args_values(input_args, "x_shift", "i", &xshift) &&
+                   grm_args_values(input_args, "y_shift", "i", &yshift) &&
+                   !grm_args_values(input_args, "movable_state", "i", &movable_status))
             {
               double ndc_xshift, ndc_yshift, rotation, tilt;
               int shift_pressed;
 
-              if (str_equals_any(kind.c_str(), 7, "wireframe", "surface", "plot3", "scatter3", "trisurface", "volume",
+              if (str_equals_any(kind, "wireframe", "surface", "plot3", "scatter3", "trisurface", "volume",
                                  "isosurface"))
                 {
                   if (grm_args_values(input_args, "shift_pressed", "i", &shift_pressed) && shift_pressed)
@@ -719,8 +860,8 @@ int grm_input(const grm_args_t *input_args)
                     }
                   else
                     {
-                      rotation = static_cast<double>(subplot_element->getAttribute("space_3d_phi"));
-                      tilt = static_cast<double>(subplot_element->getAttribute("space_3d_theta"));
+                      rotation = static_cast<double>(central_region->getAttribute("space_3d_phi"));
+                      tilt = static_cast<double>(central_region->getAttribute("space_3d_theta"));
 
                       rotation += xshift * 0.2;
                       tilt -= yshift * 0.2;
@@ -734,8 +875,8 @@ int grm_input(const grm_args_t *input_args)
                           tilt = 0;
                         }
                       grm_get_render()->setAutoUpdate(false);
-                      subplot_element->setAttribute("space_3d_phi", rotation);
-                      subplot_element->setAttribute("space_3d_theta", tilt);
+                      central_region->setAttribute("space_3d_phi", rotation);
+                      central_region->setAttribute("space_3d_theta", tilt);
                       grm_get_render()->setAutoUpdate(true);
                     }
                 }
@@ -784,73 +925,31 @@ int grm_input(const grm_args_t *input_args)
 
               if (!disable_movable_trans && movable != nullptr)
                 {
-                  double tmp = 0, x_with_shift, y_with_shift, x_ndc, y_ndc;
-                  double old_x_shift = 0, old_y_shift = 0, wc_x_shift, wc_y_shift;
-
                   movable_obj_ref = movable;
 
-                  GRM::Render::processViewport(subplot_element);
                   GRM::Render::processLimits(subplot_element);
-                  GRM::Render::processWindow(subplot_element);
+                  GRM::Render::processWindow(central_region);
 
                   gr_savestate();
                   for (const auto &elem : subplot_element->parentElement()->children())
                     {
-                      if (elem->hasAttribute("viewport_x_min"))
+                      if (elem->hasAttribute("scale"))
                         {
-                          gr_setviewport((double)elem->getAttribute("viewport_x_min"),
-                                         (double)elem->getAttribute("viewport_x_max"),
-                                         (double)elem->getAttribute("viewport_y_min"),
-                                         (double)elem->getAttribute("viewport_y_max"));
-                          gr_setwindow(
-                              (double)elem->getAttribute("window_x_min"), (double)elem->getAttribute("window_x_max"),
-                              (double)elem->getAttribute("window_y_min"), (double)elem->getAttribute("window_y_max"));
+                          gr_setviewport(static_cast<double>(central_region->getAttribute("viewport_x_min")),
+                                         static_cast<double>(central_region->getAttribute("viewport_x_max")),
+                                         static_cast<double>(central_region->getAttribute("viewport_y_min")),
+                                         static_cast<double>(central_region->getAttribute("viewport_y_max")));
+                          gr_setwindow(static_cast<double>(central_region->getAttribute("window_x_min")),
+                                       static_cast<double>(central_region->getAttribute("window_x_max")),
+                                       static_cast<double>(central_region->getAttribute("window_y_min")),
+                                       static_cast<double>(central_region->getAttribute("window_y_max")));
                           gr_setscale(static_cast<int>(elem->getAttribute("scale")));
                           break;
                         }
                     }
+                  GRM::Render::calculateCharHeight(central_region);
 
-                  x_with_shift = ndc_x + (double)xshift / max_width_height;
-                  x_ndc = ndc_x;
-                  y_with_shift = ndc_y + (double)yshift / max_width_height;
-                  y_ndc = ndc_y;
-
-                  gr_ndctowc(&x_with_shift, &y_with_shift);
-                  gr_ndctowc(&x_ndc, &y_ndc);
-
-                  wc_x_shift = x_with_shift - x_ndc;
-                  wc_y_shift = y_ndc - y_with_shift;
-
-                  if (movable->hasAttribute("x_shift"))
-                    old_x_shift = static_cast<double>(movable->getAttribute("x_shift"));
-                  if (movable->hasAttribute("y_shift"))
-                    old_y_shift = static_cast<double>(movable->getAttribute("y_shift"));
-
-                  if (xshift != 0)
-                    {
-                      if (movable->localName() == "polyline" &&
-                          static_cast<std::string>(movable->getAttribute("name")) == "integral_left")
-                        {
-                          double old_int_lim_low =
-                              static_cast<double>(movable->parentElement()->getAttribute("int_lim_low"));
-                          movable->parentElement()->setAttribute("int_lim_low", old_int_lim_low + wc_x_shift);
-                        }
-                      else if (movable->localName() == "polyline" &&
-                               static_cast<std::string>(movable->getAttribute("name")) == "integral_right")
-                        {
-                          double old_int_lim_high =
-                              static_cast<double>(movable->parentElement()->getAttribute("int_lim_high"));
-                          movable->parentElement()->setAttribute("int_lim_high", old_int_lim_high + wc_x_shift);
-                        }
-                      else
-                        {
-                          movable->setAttribute("x_shift", old_x_shift + wc_x_shift);
-                        }
-                    }
-                  if (yshift != 0)
-                    {
-                      movable->setAttribute("y_shift", old_y_shift + wc_y_shift);
-                    }
+                  moveTransformationHelper(movable, ndc_x, ndc_y, xshift, yshift, true);
                   gr_restorestate();
                 }
             }
@@ -893,23 +992,19 @@ int grm_is3d(const int x, const int y)
   double ndc_x, ndc_y;
   const char *kind;
 
-  GRM::Render::get_figure_size(&width, &height, nullptr, nullptr);
+  GRM::Render::getFigureSize(&width, &height, nullptr, nullptr);
   max_width_height = grm_max(width, height);
   ndc_x = (double)x / max_width_height;
   ndc_y = (double)y / max_width_height;
 
   auto subplot_element = get_subplot_from_ndc_points_using_dom(1, &ndc_x, &ndc_y);
 
-  if (subplot_element &&
-      str_equals_any(static_cast<std::string>(subplot_element->getAttribute("kind")).c_str(), 7, "wireframe", "surface",
-                     "plot3", "scatter3", "trisurface", "volume", "isosurface"))
+  if (subplot_element && str_equals_any(static_cast<std::string>(subplot_element->getAttribute("kind")), "wireframe",
+                                        "surface", "plot3", "scatter3", "trisurface", "volume", "isosurface"))
     {
       return 1;
     }
-  else
-    {
-      return 0;
-    }
+  return 0;
 }
 
 int grm_get_box(const int x1, const int y1, const int x2, const int y2, const int keep_aspect_ratio, int *x, int *y,
@@ -920,21 +1015,22 @@ int grm_get_box(const int x1, const int y1, const int x2, const int y2, const in
   double viewport_mid_x, viewport_mid_y;
   double ws_window[4], viewport[4];
   std::shared_ptr<GRM::Element> subplot_element;
-  GRM::Render::get_figure_size(&width, &height, nullptr, nullptr);
+  GRM::Render::getFigureSize(&width, &height, nullptr, nullptr);
   max_width_height = grm_max(width, height);
   if (!get_focus_and_factor_from_dom(x1, y1, x2, y2, keep_aspect_ratio, &factor_x, &factor_y, &focus_x, &focus_y,
                                      subplot_element))
     {
       return 0;
     }
+  auto central_region = subplot_element->querySelectors("central_region");
   ws_window[0] = static_cast<double>(subplot_element->parentElement()->getAttribute("ws_window_x_min"));
   ws_window[1] = static_cast<double>(subplot_element->parentElement()->getAttribute("ws_window_x_max"));
   ws_window[2] = static_cast<double>(subplot_element->parentElement()->getAttribute("ws_window_y_min"));
   ws_window[3] = static_cast<double>(subplot_element->parentElement()->getAttribute("ws_window_y_max"));
-  viewport[0] = static_cast<double>(subplot_element->getAttribute("viewport_x_min"));
-  viewport[1] = static_cast<double>(subplot_element->getAttribute("viewport_x_max"));
-  viewport[2] = static_cast<double>(subplot_element->getAttribute("viewport_y_min"));
-  viewport[3] = static_cast<double>(subplot_element->getAttribute("viewport_y_max"));
+  viewport[0] = static_cast<double>(central_region->getAttribute("viewport_x_min"));
+  viewport[1] = static_cast<double>(central_region->getAttribute("viewport_x_max"));
+  viewport[2] = static_cast<double>(central_region->getAttribute("viewport_y_min"));
+  viewport[3] = static_cast<double>(central_region->getAttribute("viewport_y_max"));
   viewport_mid_x = (viewport[1] + viewport[0]) / 2.0;
   viewport_mid_y = (viewport[3] + viewport[2]) / 2.0;
   *w = (int)grm_round(factor_x * width * (viewport[1] - viewport[0]) / (ws_window[1] - ws_window[0]));
@@ -1147,7 +1243,6 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
   unsigned int x_length, y_length, z_length, series_i = 0, i;
   std::string orientation;
   int is_vertical = 0;
-  int z_log = 0;
 
   auto info = static_cast<grm_tooltip_info_t *>(malloc(sizeof(grm_tooltip_info_t)));
   return_error_if(info == nullptr, ERROR_MALLOC);
@@ -1159,7 +1254,7 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
   info->ylabel = (char *)"y";
   info->label = (char *)"";
 
-  GRM::Render::get_figure_size(&width, &height, nullptr, nullptr);
+  GRM::Render::getFigureSize(&width, &height, nullptr, nullptr);
   max_width_height = grm_max(width, height);
   x = (double)mouse_x / max_width_height;
   y = (double)(height - mouse_y) / max_width_height;
@@ -1176,43 +1271,39 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
         }
     }
   if (subplot_element == nullptr ||
-      !str_equals_any(kind.c_str(), 13, "line", "scatter", "stem", "stairs", "heatmap", "marginal_heatmap", "contour",
-                      "imshow", "contourf", "pie", "hexbin", "shade", "quiver"))
+      !str_equals_any(kind, "line", "scatter", "stem", "stairs", "heatmap", "marginal_heatmap", "contour", "imshow",
+                      "contourf", "pie", "hexbin", "shade", "quiver"))
     {
       tooltip_callback(mouse_x, mouse_y, info);
       return ERROR_NONE;
     }
 
-  GRM::Render::processViewport(subplot_element);
   GRM::Render::processLimits(subplot_element);
-  GRM::Render::processWindow(subplot_element);
 
   gr_savestate();
-  if (subplot_element->hasAttribute("viewport_x_min"))
+  auto central_region = subplot_element->querySelectors("central_region");
+  GRM::Render::processWindow(central_region);
+  if (central_region->hasAttribute("window_x_min"))
     {
-      gr_setviewport((double)subplot_element->getAttribute("viewport_x_min"),
-                     (double)subplot_element->getAttribute("viewport_x_max"),
-                     (double)subplot_element->getAttribute("viewport_y_min"),
-                     (double)subplot_element->getAttribute("viewport_y_max"));
-      gr_setwindow(
-          (double)subplot_element->getAttribute("window_x_min"), (double)subplot_element->getAttribute("window_x_max"),
-          (double)subplot_element->getAttribute("window_y_min"), (double)subplot_element->getAttribute("window_y_max"));
+      gr_setviewport(static_cast<double>(central_region->getAttribute("viewport_x_min")),
+                     static_cast<double>(central_region->getAttribute("viewport_x_max")),
+                     static_cast<double>(central_region->getAttribute("viewport_y_min")),
+                     static_cast<double>(central_region->getAttribute("viewport_y_max")));
+      gr_setwindow(static_cast<double>(central_region->getAttribute("window_x_min")),
+                   static_cast<double>(central_region->getAttribute("window_x_max")),
+                   static_cast<double>(central_region->getAttribute("window_y_min")),
+                   static_cast<double>(central_region->getAttribute("window_y_max")));
     }
+  GRM::Render::calculateCharHeight(central_region);
   gr_setscale(static_cast<int>(subplot_element->getAttribute("scale")));
-  z_log = static_cast<int>(subplot_element->getAttribute("z_log"));
-
   gr_ndctowc(&x, &y);
 
   auto axes_vec = subplot_element->querySelectorsAll("axes");
+  auto coordinate_system = subplot_element->querySelectors("coordinate_system");
   std::vector<std::shared_ptr<GRM::Element>> label_vec;
-  for (const auto &child : axes_vec)
-    {
-      if (child->hasAttribute("x_label") && child->hasAttribute("y_label"))
-        {
-          label_vec.push_back(child);
-          break;
-        }
-    }
+
+  if (coordinate_system && coordinate_system->hasAttribute("x_label") && coordinate_system->hasAttribute("y_label"))
+    label_vec.push_back(coordinate_system);
 
   if (label_vec.empty())
     {
@@ -1329,22 +1420,27 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
     {
       auto current_series_group_vec = subplot_element->querySelectorsAll("series_" + kind);
       std::vector<std::shared_ptr<GRM::Element>> current_series_vec;
-      for (const auto &current_series_group : current_series_group_vec)
+      if (kind != "marginal_heatmap")
         {
-          if (str_equals_any(kind.c_str(), 7, "hexbin", "imshow", "contour", "contourf", "heatmap", "quiver", "shade"))
+          for (const auto &current_series_group : current_series_group_vec)
             {
-              current_series_vec.push_back(current_series_group);
-            }
-          else
-            {
-              for (const auto &current_series_group_child : current_series_group->children())
+              if (str_equals_any(kind, "hexbin", "imshow", "contour", "contourf", "heatmap", "quiver", "shade"))
                 {
-                  if ((kind == "marginal_heatmap" && current_series_group_child->localName() == "series_heatmap") ||
-                      kind != "marginal_heatmap")
-                    current_series_vec.push_back(current_series_group_child);
+                  current_series_vec.push_back(current_series_group);
+                }
+              else
+                {
+                  for (const auto &current_series_group_child : current_series_group->children())
+                    {
+                      current_series_vec.push_back(current_series_group_child);
+                    }
                 }
             }
         }
+      else
+        {
+          current_series_vec.push_back(subplot_element->querySelectors("marginal_heatmap_plot"));
+        }
 
       for (auto &current_series : current_series_vec)
         {
@@ -1369,7 +1465,7 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
           auto x_key = static_cast<std::string>(current_series->getAttribute(x_key_string));
           auto y_key = static_cast<std::string>(current_series->getAttribute(y_key_string));
 
-          if (!str_equals_any(kind.c_str(), 7, "hexbin", "quiver", "line", "scatter", "stem", "stairs", "shade"))
+          if (!str_equals_any(kind, "hexbin", "quiver", "line", "scatter", "stem", "stairs", "shade"))
             {
               if (!current_series->hasAttribute(z_key_string))
                 {
@@ -1394,14 +1490,13 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
             }
           std::vector<double> z_series_vec;
 
-          if (str_equals_any(kind.c_str(), 5, "heatmap", "marginal_heatmap", "contour", "imshow", "contourf"))
+          if (str_equals_any(kind, "heatmap", "marginal_heatmap", "contour", "imshow", "contourf"))
             {
               z_series_vec = GRM::get<std::vector<double>>((*context)[z_key]);
               z_length = z_series_vec.size();
             }
 
-          if (!str_equals_any(kind.c_str(), 6, "heatmap", "marginal_heatmap", "contour", "imshow", "contourf",
-                              "quiver"))
+          if (!str_equals_any(kind, "heatmap", "marginal_heatmap", "contour", "imshow", "contourf", "quiver"))
             {
               for (i = 0; i < x_series_vec.size(); i++)
                 {
@@ -1528,7 +1623,7 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
               mindiff = 0;
               x_series_idx = (mouse_x - x_0) / x_step;
               if (static_cast<int>(subplot_element->getAttribute("x_log")) &&
-                  str_equals_any(kind.c_str(), 4, "heatmap", "marginal_heatmap", "contour", "contourf"))
+                  str_equals_any(kind, "heatmap", "marginal_heatmap", "contour", "contourf"))
                 {
                   double tmp = 0;
                   x_series_idx = x_offset;
@@ -1542,7 +1637,7 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
 
               y_series_idx = (mouse_y - y_0) / y_step;
               if (static_cast<int>(subplot_element->getAttribute("y_log")) &&
-                  str_equals_any(kind.c_str(), 4, "heatmap", "marginal_heatmap", "contour", "contourf"))
+                  str_equals_any(kind, "heatmap", "marginal_heatmap", "contour", "contourf"))
                 {
                   double tmp = 0;
                   y_series_idx = y_offset;
@@ -1636,7 +1731,7 @@ int grm_get_hover_mode(int mouse_x, int mouse_y, int disable_movable_xform)
   if (!disable_movable_xform)
     {
       int width, height, max_width_height;
-      GRM::Render::get_figure_size(&width, &height, nullptr, nullptr);
+      GRM::Render::getFigureSize(&width, &height, nullptr, nullptr);
       max_width_height = grm_max(width, height);
 
       double ndc_x, ndc_y;
